@@ -1,54 +1,57 @@
 #!/usr/bin/perl -T
-#
-# openwebmail-viewatt.pl - attachment reading program
-#
+#############################################################################
+# Open WebMail - Provides a web interface to user mailboxes                 #
+#                                                                           #
+# Copyright (C) 2001-2002                                                   #
+# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric                 #
+# Copyright (C) 2000                                                        #
+# Ernie Miller  (original GPL project: Neomail)                             #
+#                                                                           #
+# This program is distributed under GNU General Public License              #
+#############################################################################
 
-use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+local $SCRIPT_DIR="";
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
-push (@INC, $SCRIPT_DIR, ".");
-
-$ENV{PATH} = ""; # no PATH should be needed
-$ENV{ENV} = "";      # no startup script for sh
-$ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0002); # make sure the openwebmail group can write
 
 use strict;
+no strict 'vars';
 use Fcntl qw(:DEFAULT :flock);
-use CGI qw(-private_tempfiles :standard);
+use CGI qw(:standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "ow-shared.pl";
+$ENV{PATH} = ""; # no PATH should be needed
+$ENV{BASH_ENV} = ""; # no startup script for bash
+umask(0007); # make sure the openwebmail group can write
+
+push (@INC, $SCRIPT_DIR, ".");
+require "openwebmail-shared.pl";
 require "filelock.pl";
 require "mime.pl";
-require "iconv.pl";
 require "maildb.pl";
 
-use vars qw(%config %config_raw);
-use vars qw($thissession);
-use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
-use vars qw(%prefs %style);
-use vars qw($folderdir @validfolders $folderusage);
-use vars qw($folder $printfolder $escapedfolder);
+local (%config, %config_raw);
+local $thissession;
+local ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir);
+local (%prefs, %style);
+local ($lang_charset, %lang_folders, %lang_sortlabels, %lang_text, %lang_err);
+local ($folderdir, @validfolders, $folderusage);
+local ($folder, $printfolder, $escapedfolder);
 
 openwebmail_init();
 verifysession();
 
-use vars qw($firstmessage);
-use vars qw($sort);
-use vars qw($searchtype $keyword $escapedkeyword);
+local $firstmessage;
+local $sort;
+local ($searchtype, $keyword, $escapedkeyword);
 
 $firstmessage = param("firstmessage") || 1;
-$sort = param("sort") || $prefs{'sort'} || 'date';
+$sort = param("sort") || $prefs{"sort"} || 'date';
 $keyword = param("keyword") || '';
 $escapedkeyword = escapeURL($keyword);
 $searchtype = param("searchtype") || 'subject';
 
-# extern vars
-use vars qw(%lang_err);			# defined in lang/xy
-use vars qw($_SUBJECT $_CHARSET);	# defined in maildb.pl
-
 ########################## MAIN ##############################
 
 my $action = param("action");
@@ -71,10 +74,7 @@ sub viewattachment {	# view attachments inside a message
 
    filelock($folderfile, LOCK_SH|LOCK_NB) or
       openwebmailerror("$lang_err{'couldnt_locksh'} $folderfile!");
-   if (update_headerdb($headerdb, $folderfile)<0) {
-      filelock($folderfile, LOCK_UN);
-      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
-   }
+   update_headerdb($headerdb, $folderfile);
    open($folderhandle, "$folderfile");
    my $r_block= get_message_block($messageid, $headerdb, $folderhandle);
    close($folderhandle);
@@ -84,27 +84,19 @@ sub viewattachment {	# view attachments inside a message
       printheader();
       $messageid = str2html($messageid);
       print "What the heck? Message $messageid seems to be gone!";
-      printfooter(1);
+      printfooter();
       return;
    }
 
-   my @attr=get_message_attributes($messageid, $headerdb);
-   my $convfrom=param('convfrom');
-   if ($convfrom eq "") {
-      if ( is_convertable($attr[$_CHARSET], $prefs{'charset'}) ) {
-         $convfrom=lc($attr[$_CHARSET]);
-      } else {
-         $convfrom='none.prefscharset';
-      }
-   }
-
-   if ( $nodeid eq 'all' ) {
+   if ( $nodeid eq 'all' ) { 
       # return whole msg as an message/rfc822 object
-      my $subject = $attr[$_SUBJECT];
-      if (is_convertable($convfrom, $prefs{'charset'}) ) {
-         ($subject)=iconv($convfrom, $prefs{'charset'}, $subject);
-      }
+      my $subject = (get_message_attributes($messageid, $headerdb))[$_SUBJECT];
       $subject =~ s/\s+/_/g;
+      if ( param('zhconvert') eq 'b2g' ) {
+         $subject= b2g($subject);
+      } elsif ( param('zhconvert') eq 'g2b' ) {
+         $subject= g2b($subject);
+      }
 
       my $length = length(${$r_block});
       # disposition:attachment default to save
@@ -133,13 +125,10 @@ sub viewattachment {	# view attachments inside a message
       }
 
       if (defined($r_attachment)) {
-         my $charset=${$r_attachment}{filenamecharset}||
-                     ${$r_attachment}{charset}||
-                     $convfrom||
-                     $attr[$_CHARSET];
-         if (is_convertable($charset, $prefs{'charset'})) {
-            (${$r_attachment}{filename})=iconv($charset, $prefs{'charset'},
-                                                ${$r_attachment}{filename});
+         if ( param('zhconvert') eq 'b2g' ) {
+            ${$r_attachment}{filename}=b2g(${$r_attachment}{filename});
+         } elsif ( param('zhconvert') eq 'g2b' ) {
+            ${$r_attachment}{filename}=g2b(${$r_attachment}{filename});
          }
 
          my $content;
@@ -160,25 +149,15 @@ sub viewattachment {	# view attachments inside a message
             $content = html4disablejs($content) if ($prefs{'disablejs'}==1);
             $content = html4disableembcgi($content) if ($prefs{'disableembcgi'}==1);
             $content = html4attachments($content, $r_attachments, "$config{'ow_cgiurl'}/openwebmail-viewatt.pl", "action=viewattachment&amp;sessionid=$thissession&amp;message_id=$escapedmessageid&amp;folder=$escapedfolder");
-#            $content = html4mailto($content, "$config{'ow_cgiurl'}/openwebmail-send.pl", "action=composemessage&amp;sort=$sort&amp;keyword=$escapedkeyword&amp;searchtype=$searchtype&amp;folder=$escapedfolder&amp;firstmessage=$firstmessage&amp;sessionid=$thissession&amp;composetype=sendto");
+            $content = html4mailto($content, "$config{'ow_cgiurl'}/openwebmail-send.pl", "action=composemessage&amp;sort=$sort&amp;keyword=$escapedkeyword&amp;searchtype=$searchtype&amp;folder=$escapedfolder&amp;firstmessage=$firstmessage&amp;sessionid=$thissession&amp;composetype=sendto");
          }
 
          my $length = length($content);
-         my $contenttype = ${$r_attachment}{contenttype};
-         my $filename = ${$r_attachment}{filename}; 
-         $filename=~s/\s$//;
+         my $contenttype = ${$r_attachment}{contenttype}; 
+         my $filename = ${$r_attachment}{filename};
 
          # remove char disallowed in some fs
-         if ($prefs{'charset'} eq 'big5') {
-            $filename =~ s|([^\xA1-\xF9])\\|$1_|;	# dos path
-         } elsif ($prefs{'charset'} eq 'gb2312') {
-            $filename =~ s|([^\x81-\xFE])\\|$1_|;	# dos path
-         } else {
-            $filename =~ s|\\|_|;			# dos path
-         }
-         $filename =~ s|^.*/||;	# unix path
-         $filename =~ s|^.*:||;	# mac path and dos drive
-
+         $filename=~s/[\s\\\/:]/_/g;	
          # IE6 will crash if filename longer than 45, tricky!
          if (length($filename)>45) {
             $filename=~/^(.*)(\.[^\.]*)$/;
@@ -192,22 +171,20 @@ sub viewattachment {	# view attachments inside a message
             $contenttype = "text/plain";
          }
 
-         # we change the filename of an attachment
-         # from *.exe, *.com *.bat, *.pif, *.lnk, *.scr to *.file
+         # we change the filename of an attachment 
+         # from *.exe, *.com *.pif, *.lnk, *.scr to *.file
          # if its contenttype is not application/octet-stream
          # to avoid this attachment is referenced by html and executed directly ie
-         if ( $filename =~ /\.(exe|com|bat|pif|lnk|scr)$/i &&
-              $contenttype !~ /application\/octet\-stream/i &&
-              $contenttype !~ /application\/x\-msdownload/i ) {
+         if ( ( $filename =~ /\.exe$/i || 
+                $filename =~ /\.com$/i ||
+                $filename =~ /\.pif$/i || 
+                $filename =~ /\.lnk$/i ||
+                $filename =~ /\.scr$/i )  &&
+               $contenttype !~ /application\/octet\-stream/i &&
+               $contenttype !~ /application\/x\-msdownload/i ) {
             $filename="$filename.file";
          }
 
-         # change contenttype of image to make it directly displayed by browser
-         if ( $contenttype =~ /application\/octet\-stream/i &&
-              $filename =~ /\.(jpg|jpeg|gif|png|bmp)$/i ) {
-            $contenttype="image/".lc($1);
-         }
-
          # disposition:attachment default to save
          print qq|Content-Length: $length\n|,
                qq|Content-Transfer-Coding: binary\n|,
@@ -233,7 +210,7 @@ sub viewattachment {	# view attachments inside a message
          printheader();
          $messageid = str2html($messageid);
          print "What the heck? Message $messageid attachmment $nodeid seems to be gone!";
-         printfooter(1);
+         printfooter();
       }
       return;
 
@@ -242,25 +219,25 @@ sub viewattachment {	# view attachments inside a message
 ################### END VIEWATTACHMENT ##################
 
 ################ VIEWATTFILE ##################
-sub viewattfile {	# view attachments uploaded to $config{'ow_sessionsdir'}
+sub viewattfile {	# view attachments uploaded to openwebmail/etc/sessions/
    my $attfile=param("attfile");
    $attfile =~ s/\///g;  # just in case someone gets tricky ...
    # only allow to view attfiles belongs the $thissession
-   if ($attfile!~/^$thissession/  || !-f "$config{'ow_sessionsdir'}/$attfile") {
+   if ($attfile!~/^$thissession/  || !-f "$config{'ow_etcdir'}/sessions/$attfile") {
       printheader();
-      print "What the heck? Attfile $config{'ow_sessionsdir'}/$attfile seems to be gone!";
-      printfooter(1);
+      print "What the heck? Attfile $config{'ow_etcdir'}/sessions/$attfile seems to be gone!";
+      printfooter();
       return;
    }
 
    my ($attsize, $attheader, $attheaderlen, $attcontent);
-   my ($attcontenttype, $attencoding, $attdisposition,
+   my ($attcontenttype, $attencoding, $attdisposition, 
        $attid, $attlocation, $attfilename);
+   
+   $attsize=(-s("$config{'ow_etcdir'}/sessions/$attfile"));
 
-   $attsize=(-s("$config{'ow_sessionsdir'}/$attfile"));
-
-   open(ATTFILE, "$config{'ow_sessionsdir'}/$attfile") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$attfile!");
+   open(ATTFILE, "$config{'ow_etcdir'}/sessions/$attfile") or 
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions/$attfile!");
    read(ATTFILE, $attheader, 512);
    $attheaderlen=index($attheader,  "\n\n", 0);
    $attheader=substr($attheader, 0, $attheaderlen);
@@ -283,7 +260,7 @@ sub viewattfile {	# view attachments uploaded to $config{'ow_sessionsdir'}
          $attdisposition = $1;
          $lastline = 'NONE';
       } elsif (/^content-id:\s+(.+)$/ig) {
-         $attid = $1;
+         $attid = $1; 
          $attid =~ s/^\<(.+)\>$/$1/;
          $lastline = 'NONE';
       } elsif (/^content-location:\s+(.+)$/ig) {
@@ -305,16 +282,7 @@ sub viewattfile {	# view attachments uploaded to $config{'ow_sessionsdir'}
    $attdisposition =~ s/^(.+);.*/$1/g;
 
    # remove char disallowed in some fs
-   if ($prefs{'charset'} eq 'big5') {
-      $attfilename =~ s|([^\xA1-\xF9])\\|$1_|;	# dos path
-   } elsif ($prefs{'charset'} eq 'gb2312') {
-      $attfilename =~ s|([^\x81-\xFE])\\|$1_|;	# dos path
-   } else {
-      $attfilename =~ s|\\|_|;			# dos path
-   }
-   $attfilename =~ s|^.*/||;	# unix path
-   $attfilename =~ s|^.*:||;	# mac path and dos drive
-
+   $attfilename=~s/[\s\\\/:]/_/g;	
    # IE6 will crash if filename longer than 45, tricky!
    if (length($attfilename)>45) {
       $attfilename=~/^(.*)(\.[^\.]*)$/;
