@@ -1,146 +1,93 @@
 #!/usr/bin/perl -T
-#################################################################
-#                                                               #
-# Open WebMail - Provides a web interface to user mailboxes     #
-#                                                               #
-# Copyright (C) 2001-2002                                       #
-# The Open Webmail Team                                         #
-#                                                               #
-# Copyright (C) 2000                                            #
-# Ernie Miller  (original GPL project: Neomail)                 #
-#                                                               #
-# This program is distributed under GNU General Public License  #
-#                                                               #
-#################################################################
+#############################################################################
+# Open WebMail - Provides a web interface to user mailboxes                 #
+#                                                                           #
+# Copyright (C) 2001-2002                                                   #
+# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric                 #
+# Copyright (C) 2000                                                        #
+# Ernie Miller  (original GPL project: Neomail)                             #
+#                                                                           #
+# This program is distributed under GNU General Public License              #
+#############################################################################
 
-#
-# openwebmail.pl - entry point of openwebmail
-#
-
-use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
+local $SCRIPT_DIR="";
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
-push (@INC, $SCRIPT_DIR, ".");
-
-$ENV{PATH} = ""; # no PATH should be needed
-$ENV{ENV} = "";      # no startup script for sh
-$ENV{BASH_ENV} = ""; # no startup script for bash
-umask(0002); # make sure the openwebmail group can write
 
 use strict;
+no strict 'vars';
 use Fcntl qw(:DEFAULT :flock);
 use Socket;
-use CGI qw(-private_tempfiles :standard);
+use CGI qw(:standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "ow-shared.pl";
+$ENV{PATH} = ""; # no PATH should be needed
+$ENV{BASH_ENV} = ""; # no startup script for bash
+umask(0007); # make sure the openwebmail group can write
+
+push (@INC, $SCRIPT_DIR, ".");
+require "openwebmail-shared.pl";
 require "filelock.pl";
 
-use vars qw(%config %config_raw);
-use vars qw($thissession);
-use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
-use vars qw(%prefs %style);
-use vars qw($folderdir);
-
-# extern vars
-use vars qw(%lang_text %lang_err);	# defined in lang/xy
-use vars qw($pop3_authserver);	# defined in auth_pop3.pl
+local (%config, %config_raw);
+local $thissession;
+local ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir);
+local (%prefs, %style);
+local ($lang_charset, %lang_folders, %lang_sortlabels, %lang_text, %lang_err);
+local $folderdir;
 
 readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/openwebmail.conf.default");
 readconf(\%config, \%config_raw, "$SCRIPT_DIR/etc/openwebmail.conf") if (-f "$SCRIPT_DIR/etc/openwebmail.conf");
 
+if ( $config{'logfile'} ne 'no' && ! -f $config{'logfile'} ) {
+   my $mailgid=getgrnam('mail');
+   open (LOGFILE,">>$config{'logfile'}") or 
+      openwebmailerror("Can't open log file $config{'logfile'}!");
+   close(LOGFILE);
+   chmod(0660, $config{'logfile'});
+   chown($>, $mailgid, $config{'logfile'});
+}
+
 # setuid is required if mails is located in user's dir
 if ( $>!=0 && ($config{'use_homedirspools'}||$config{'use_homedirfolders'}) ) {
    print "Content-type: text/html\n\n'$0' must setuid to root"; exit 0;
 }
 
-# check & create mapping table for solar/lunar, b2g, g2b convertion
-if ( ! -f "$config{'ow_etcdir'}/lunar$config{'dbm_ext'}" ||
-     ! -f "$config{'ow_etcdir'}/g2b$config{'dbm_ext'}" ||
-     ! -f "$config{'ow_etcdir'}/b2g$config{'dbm_ext'}" ) {
-   print "Content-type: text/html\n\nPlease execute  '$config{'ow_cgidir'}/openwebmail-tool.pl --init' on server first!"; exit 0;
-}
+%prefs = %{&readprefs};
+%style = %{&readstyle};
 
-# validate allowed_serverdomain
-my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
-if (! is_serverdomain_allowed($httphost) ) {
-   print "Content-type: text/html\n\nService is not available for domain  ' $httphost '";
-   exit 0;
-}
-
-if ( ($config{'logfile'} ne 'no') ) {
-   my $mailgid=getgrnam('mail');
-   my ($fmode, $fuid, $fgid) = (stat($config{'logfile'}))[2,4,5];
-   if ( !($fmode & 0100000) ) {
-      open (LOGFILE,">>$config{'logfile'}") or
-         openwebmailerror("Can't open log file $config{'logfile'}!");
-      close(LOGFILE);
-   }
-   chmod(0660, $config{'logfile'}) if (($fmode&0660)!=0660);
-   chown($>, $mailgid, $config{'logfile'}) if ($fuid!=$>||$fgid!=$mailgid);
-}
+($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+require "etc/lang/$prefs{'language'}";
+$lang_charset ||= 'iso-8859-1';
 
 ####################### MAIN ##########################
-
 if ( param("loginname") && param("password") ) {
    $loginname=param("loginname");
-   $loginname.='@'.param("logindomain") if ($loginname!~/\@/ && param("logindomain") ne "");
-   if ($config{'case_insensitive_login'}) {
-      $loginname=lc($loginname);
-   } else {
-      $loginname=$1.'@'.lc($2) if ($loginname=~/^(.+)\@(.+)$/);
-   }
-
    my $siteconf;
    if ($loginname=~/\@(.+)$/) {
-       my $domain=$1;
-       if (! is_serverdomain_allowed($domain)) {
-          openwebmailerror("Service is not available for domain  ' $domain '");
-       }
-       $siteconf="$config{'ow_sitesconfdir'}/$domain";
+       $siteconf="$config{'ow_etcdir'}/sites.conf/$1";
    } else {
-       my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
-       $siteconf="$config{'ow_sitesconfdir'}/$httphost";
+       $siteconf="$config{'ow_etcdir'}/sites.conf/$ENV{'HTTP_HOST'}";
    }
-   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
+   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf"); 
 
    require $config{'auth_module'} or
       openwebmailerror("Can't open authentication module $config{'auth_module'}");
 
-   if ( ($config{'logfile'} ne 'no') ) {
+   if ( ($config{'logfile'} ne 'no') && (! -f $config{'logfile'})  ) {
       my $mailgid=getgrnam('mail');
-      my ($fmode, $fuid, $fgid) = (stat($config{'logfile'}))[2,4,5];
-      if ( !($fmode & 0100000) ) {
-         open (LOGFILE,">>$config{'logfile'}") or
-            openwebmailerror("Can't open log file $config{'logfile'}!");
-         close(LOGFILE);
-      }
-      chmod(0660, $config{'logfile'}) if (($fmode&0660)!=0660);
-      chown($>, $mailgid, $config{'logfile'}) if ($fuid!=$>||$fgid!=$mailgid);
+      open (LOGFILE,">>$config{'logfile'}") or 
+         openwebmailerror("Can't open log file $config{'logfile'}!");
+      close(LOGFILE);
+      chmod(0660, $config{'logfile'});
+      chown($>, $mailgid, $config{'logfile'});
    }
 
-   my $virtname=$config{'virtusertable'};
-   $virtname=~s!/!.!g; $virtname=~s/^\.+//;
-   update_virtusertable("$config{'ow_etcdir'}/$virtname", $config{'virtusertable'});
-
-   %prefs = %{&readprefs};
-   %style = %{&readstyle};
-   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
-   require "$config{'ow_langdir'}/$prefs{'language'}";
+   update_virtusertable("$config{'ow_etcdir'}/virtusertable", $config{'virtusertable'});
 
    login();
-
 } else {            # no action has been taken, display login page
-   my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
-   my $siteconf="$config{'ow_sitesconfdir'}/$httphost";
-   readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
-
-   %prefs = %{&readprefs};
-   %style = %{&readstyle};
-   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
-   require "$config{'ow_langdir'}/$prefs{'language'}";
-
    loginmenu();
 }
 
@@ -149,63 +96,35 @@ exit 0;
 
 ##################### LOGINMENU ######################
 sub loginmenu {
+   printheader(),
    my $html='';
    my $temphtml;
+   open (LOGIN, "$config{'ow_etcdir'}/templates/$prefs{'language'}/login.template") or
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/login.template!");
+   while (<LOGIN>) {
+      $html .= $_;
+   }
+   close (LOGIN);
 
-   $html=readtemplate("login.template");
    $html = applystyle($html);
 
-   printheader(),
-
    $temphtml = startform(-action=>"$config{'ow_cgiurl'}/openwebmail.pl",
                          -name=>'login');
-   if (defined(param('action'))) {
-      $temphtml .= hidden("action", param('action'));
-      $temphtml .= hidden("to", param('to')) if (defined(param('to')));
-      $temphtml .= hidden("subject", param('subject')) if (defined(param('subject')));
-   }
+   $temphtml .= hidden("action","login");
    $html =~ s/\@\@\@STARTFORM\@\@\@/$temphtml/;
-
-   if (cookie("openwebmail-rememberme")) {
-      $temphtml = textfield(-name=>'loginname',
-                            -default=>cookie("openwebmail-loginname"),
-                            -size=>'12',
-                            -onChange=>'focuspwd()',
-                            -override=>'1');
-   } else {
-      $temphtml = textfield(-name=>'loginname',
-                            -default=>'' ,
-                            -size=>'12',
-                            -onChange=>'focuspwd()',
-                            -override=>'1');
-   }
-   $html =~ s/\@\@\@LOGINNAMEFIELD\@\@\@/$temphtml/;
-
+   $temphtml = textfield(-name=>'loginname',
+                         -default=>'',
+                         -size=>'10',
+                         -onChange=>'focuspwd()', 
+                         -override=>'1');
+   $html =~ s/\@\@\@USERIDFIELD\@\@\@/$temphtml/;
    $temphtml = password_field(-name=>'password',
                               -default=>'',
-                              -size=>'12',
-                              -onChange=>'focusloginbutton()',
+                              -size=>'10',
+                              -onChange=>'focuslogin()', 
                               -override=>'1');
    $html =~ s/\@\@\@PASSWORDFIELD\@\@\@/$temphtml/;
-
-   $temphtml = checkbox(-name=>'rememberme',
-                        -value=>'1',
-                        -checked=>cookie("openwebmail-rememberme")||0,
-                        -label=>'');
-   $html =~ s/\@\@\@REMEMBERMECHECKBOX\@\@\@/$temphtml/;
-
-   if ( $#{$config{'domainnames'}} >0 && $config{'enable_domainselectmenu'} ) {
-      $temphtml = popup_menu(-name=>'logindomain',
-                             -values=>[@{$config{'domainnames'}}] );
-      $html =~ s/\@\@\@DOMAINMENU\@\@\@/$temphtml/;
-      $html =~ s/\@\@\@DOMAINSTART\@\@\@//;
-      $html =~ s/\@\@\@DOMAINEND\@\@\@//;
-   } else {
-      $html =~ s/\@\@\@DOMAINSTART\@\@\@/<!--/;
-      $html =~ s/\@\@\@DOMAINEND\@\@\@/-->/;
-   }
-
-   $temphtml = submit(-name =>"loginbutton",
+   $temphtml = submit(-name =>"login",
 		      -value=>"$lang_text{'login'}" );
 
    $html =~ s/\@\@\@LOGINBUTTON\@\@\@/$temphtml/;
@@ -227,18 +146,10 @@ sub login {
    $password = $1;
 
    ($loginname, $domain, $user, $userrealname, $uuid, $ugid, $homedir)
-					=get_domain_user_userinfo($loginname);
-
-   ($user =~ /^(.+)$/) && ($user = $1);		# untaint...
-   ($uuid =~ /^(.+)$/) && ($uuid = $1);
-   ($ugid =~ /^(.+)$/) && ($ugid = $1);
-   ($homedir =~ /^(.+)$/) && ($homedir = $1);
-
+	=get_domain_user_userinfo($loginname);
    if ($user eq "") {
       sleep $config{'loginerrordelay'};	# delayed response
-      writelog("login error - no such user - loginname=$loginname");
-      # show 'pwd incorrect' instead of 'user not exist' for better security
-      openwebmailerror("$lang_err{'pwd_incorrect'}");	
+      openwebmailerror("$lang_err{'user_not_exist'}");
    }
    if (! $config{'enable_rootlogin'}) {
       if ($user eq 'root' || $uuid==0) {
@@ -252,21 +163,26 @@ sub login {
    if ($config{'auth_withdomain'}) {
       $errorcode=check_userpassword("$user\@$domain", $password);
    } else {
-      $errorcode=check_userpassword($user, $password);
+      if ($domain eq $ENV{'HTTP_HOST'}) {
+         $errorcode=check_userpassword($user, $password);
+      } else {
+         sleep $config{'loginerrordelay'};	# delayed response
+         openwebmailerror("$lang_err{'user_not_exist'}");
+      }
    }
    if ( $errorcode==0 ) {
-      my $userconf="$config{'ow_usersconfdir'}/$user";
+      my $userconf="$config{'ow_etcdir'}/users.conf/$user";
       $userconf .= "\@$domain" if ($config{'auth_withdomain'});
-      readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf");
+      readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf"); 
 
       my $clientip=get_clientip();
       # validate client ip
       if ($#{$config{'allowed_clientip'}}>=0) {
          my $allowed=0;
          foreach my $token (@{$config{'allowed_clientip'}}) {
-            if (lc($token) eq 'all' || $clientip=~/^\Q$token\E/) {
+            if ($token eq 'ALL' || $clientip=~/^\Q$token\E/) {
                $allowed=1; last;
-            } elsif (lc($token) eq 'none') {
+            } elsif ($token eq 'NONE') {
                last;
             }
          }
@@ -279,15 +195,15 @@ sub login {
          my $clientdomain;
          my $allowed=0;
          foreach my $token (@{$config{'allowed_clientdomain'}}) {
-            if (lc($token) eq 'all') {
+            if ($token eq 'ALL') {
                $allowed=1; last;
-            } elsif (lc($token) eq 'none') {
+            } elsif ($token eq 'NONE') {
                last;
             }
             $clientdomain=ip2hostname($clientip) if ($clientdomain eq "");
             if ($clientdomain=~/\Q$token\E$/ || # matched
                 $clientdomain!~/\./) { 		# shortname in /etc/hosts
-               $allowed=1; last;
+               $allowed=1; last; 
             }
          }
          if (!$allowed) {
@@ -295,27 +211,13 @@ sub login {
          }
       }
 
-      # search old alive session and deletes old expired sessionids
-      $thissession = search_and_cleanoldsessions(cookie("$user-sessionid"));
-      if ($thissession eq "") {
-         $thissession = $loginname. "-session-" . rand(); # name the sessionid
-      }
+      $thissession = $loginname. "-session-" . rand(); # name the sessionid
       writelog("login - $thissession");
-      ($thissession =~ /^(.+)$/) && ($thissession = $1);  # untaint
-
-      # create the user's home directory if necessary.
-      # this must be done before changing to the user's uid.
-      if ( $config{'create_homedir'} && ! -d "$homedir" ) {
-         if (mkdir ("$homedir", oct(700)) and chown($uuid, $ugid, $homedir)) {
-            writelog("mkdir - $homedir, uid=$uuid, gid=$ugid");
-         } else {
-            openwebmailerror("$lang_err{'cant_create_dir'} $homedir");
-         }
-      }
+      cleanupoldsessions(); # Deletes sessionids that have expired
 
       if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
          my $mailgid=getgrnam('mail');
-         set_euid_egid_umask($uuid, $mailgid, 0077);
+         set_euid_egid_umask($uuid, $mailgid, 0077);	
          if ( $) != $mailgid) {	# egid must be mail since this is a mail program...
             openwebmailerror("Set effective gid to mail($mailgid) failed!");
          }
@@ -324,43 +226,40 @@ sub login {
       if ( $config{'use_homedirfolders'} ) {
          $folderdir = "$homedir/$config{'homedirfolderdirname'}";
       } else {
-         $folderdir = "$config{'ow_usersdir'}/$user";
+         $folderdir = "$config{'ow_etcdir'}/users/$user";
          $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
       }
-      ($folderdir =~ /^(.+)$/) && ($folderdir = $1);	# untaint
+
+      ($thissession =~ /^(.+)$/) && ($thissession = $1);  # untaint ...
+      ($user =~ /^(.+)$/) && ($user = $1);
+      ($uuid =~ /^(.+)$/) && ($uuid = $1);
+      ($ugid =~ /^(.+)$/) && ($ugid = $1);
+      ($homedir =~ /^(.+)$/) && ($homedir = $1);
+      ($folderdir =~ /^(.+)$/) && ($folderdir = $1);
 
       # create folderdir if it doesn't exist
       if (! -d "$folderdir" ) {
-         if (mkdir ("$folderdir", oct(700))) {
-            writelog("mkdir - $folderdir, euid=$>, egid=$)");
-         } else {
+         mkdir ("$folderdir", oct(700)) or
             openwebmailerror("$lang_err{'cant_create_dir'} $folderdir");
-         }
       }
 
       # create system spool file /var/mail/xxxx
       my ($spoolfile, $headerdb)=get_folderfile_headerdb($user, 'INBOX');
       if ( ! -f "$spoolfile" ) {
-         ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); # untaint ...
+         ($spoolfile =~ /^(.+)$/) && ($spoolfile = $1); # bypass taint check
          open (F, ">>$spoolfile"); close(F);
          chown($uuid, $ugid, $spoolfile);
       }
 
       # create session file
-      my $sessioncookie_value;
-      if ( -f "$config{'ow_sessionsdir'}/$thissession" ) { # continue an old session?
-         $sessioncookie_value = cookie("$user-sessionid");
-      } else {						       # a brand new sesion?
-         $sessioncookie_value = crypt(rand(),'OW');
-      }
-      open (SESSION, "> $config{'ow_sessionsdir'}/$thissession") or # create sessionid
-         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$thissession!");
-      print SESSION $sessioncookie_value, "\n";
-      print SESSION get_clientip(), "\n";
+      my $setcookie = crypt(rand(),'OW');
+      open (SESSION, "> $config{'ow_etcdir'}/sessions/$thissession") or # create sessionid
+         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions/$thissession!");
+      print SESSION $setcookie;
       close (SESSION);
       writehistory("login - $thissession");
 
-      # symbolic link ~/mbox to ~/mail/saved-messages
+      # symbolic link ~/mbox to ~/mail/saved-messages 
       if ( $config{'symboliclink_mbox'} &&
            ((lstat("$homedir/mbox"))[2] & 07770000) eq 0100000) { # regular file
          if (filelock("$folderdir/saved-messages", LOCK_EX|LOCK_NB)) {
@@ -415,7 +314,7 @@ sub login {
             }
 
             if ($pop3host ne $pop3_authserver ||
-                $pop3user ne $login ||
+                $pop3user ne $login || 
                 $pop3pass ne $password) {
                if ($pop3host ne $pop3_authserver || $pop3user ne $login) {
                   $pop3lastid="none";
@@ -437,61 +336,27 @@ sub login {
          $url="$config{'ow_cgiurl'}/openwebmail-prefs.pl?sessionid=$thissession&action=firsttimeuser";
       } elsif ( $rc_upgrade ) {
          $url="$config{'ow_cgiurl'}/openwebmail-prefs.pl?sessionid=$thissession&action=editprefs";
-      } elsif ( param('action') eq 'composemessage' ) {
+      } elsif ( defined(param("to")) ) {
          my $to=param("to");
          my $subject=param("subject");
          $url="$config{'ow_cgiurl'}/openwebmail-send.pl?sessionid=$thissession&action=composemessage&to=$to&subject=$subject";
-      } elsif ( param('action') eq 'calyear' || param('action') eq 'calmonth' ||
-                param('action') eq 'calweek' || param('action') eq 'calday' ) {
-         my $action=param('action');
-         $url="$config{'ow_cgiurl'}/openwebmail-cal.pl?sessionid=$thissession&action=$action";
       } else {
          $url="$config{'ow_cgiurl'}/openwebmail-main.pl?sessionid=$thissession&action=displayheaders_afterlogin";
       }
 
-      if ( ($ENV{'HTTPS'}=~/on/i || $ENV{'SERVER_PORT'}==443) && !$config{'stay_ssl_afterlogin'}) {
-         $url="http://$ENV{'HTTP_HOST'}$url" if ($url !~ m!https?://! );
-      }
-
       my @headers=();
       push(@headers, -pragma=>'no-cache');
-
-      # cookie for openwebmail to verify session, expired if browser closed
-      my $cookie1 = cookie( -name  => "$user-sessionid",
-                            -value => "$sessioncookie_value",
-                            -path  => '/');
-      # cookie for other ap to find openwebmail loginname, expired until 1 month later
-      my $cookie2 = cookie( -name  => "openwebmail-loginname",
-                            -value => "$loginname",
-                            -path  => '/',
-                            -expires => "+1M" );
-      # cookie for remember loginname switch, expired until 1 month later
-      my $cookie3 = cookie( -name  => "openwebmail-rememberme",
-                            -value => param("rememberme")||'',
-                            -path  => '/',
-                            -expires => "+1M" );
-      # cookie for ssl session, expired if not same session
-      my $cookie4 = cookie( -name  => "openwebmail-ssl",
-                            -value => ($ENV{'HTTPS'}=~/on/i ||
-                                       $ENV{'SERVER_PORT'}==443 ||
-                                       0),
-                            -path  => '/');
-
-      push(@headers, -cookie=>[$cookie1, $cookie2, $cookie3, $cookie4]);
-      push(@headers, -charset=>$prefs{'charset'}) if ($CGI::VERSION>=2.57);
-
-      # load page with Refresh header only if not MSIE on Mac
-      my $refresh;
-      if ($ENV{'HTTP_USER_AGENT'} !~ /MSIE.+Mac/) {
-         # push(@headers, -Refresh=>"0;URL=$url");
-         $refresh=qq|<meta http-equiv="refresh" content="0;URL=$url">|;
-      }
-
+      my $cookie = cookie( -name    => "$user-sessionid",
+                           -value   => "$setcookie",
+                           -path    => '/' );
+      push(@headers, -cookie=>$cookie);
+      push(@headers, -charset=>$lang_charset) if ($CGI::VERSION>=2.57);
+      push(@headers, -Refresh=>"0;URL=$url");
       print header(@headers);
 
       # display copyright. Don't touch it, please.
       print	qq|<html>\n|,
-		qq|<head><title>Copyright</title>$refresh</head>\n|,
+		qq|<head><title>Copyright</title></head>\n|,
 		qq|<body bgcolor="#ffffff" background="$prefs{'bgurl'}">\n|,
 		qq|<style type="text/css"><!--\n|,
 		qq|body { background-image: url($prefs{'bgurl'}); background-repeat: no-repeat; }\n|,
@@ -504,7 +369,7 @@ sub login {
 		qq|<font color="#cccccc" face="arial,helvetica,sans-serif" size=-1>\n|,
                 qq|$config{'name'} $config{'version'} $config{'releasedate'}<br><br>\n|,
 		qq|Copyright (C) 2001-2002<br>\n|,
-		qq|Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung, Dattola Filippo<br><br>\n|,
+		qq|Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric<br><br>\n|,
 		qq|Copyright (C) 2000<br>\n|,
 		qq|Ernie Miller  (original GPL project: Neomail)<br><br>\n|,
 		qq|</font></a>\n\n|,
@@ -518,16 +383,11 @@ sub login {
 		qq|MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.<br>\n|,
 		qq|See the GNU General Public License for more details.<br><br>\n|,
 		qq|Removal or change of this copyright is prohibited.\n|,
-		qq|</font></a>\n\n|;
-
-      # load page with java script if MSIE on Mac
-      if ($ENV{'HTTP_USER_AGENT'}=~/MSIE.+Mac/) {
-         print  qq|<script language="JavaScript">\n<!--\n|,
-		qq|setTimeout("window.open('$url','_self')", 1*1000);\n|,
-		qq|//-->\n</script>\n|;
-      }
-
-      print     qq|</center></body></html>\n|;
+		qq|</font></a>\n\n|,
+                qq|<script language="JavaScript">\n<!--\n|,
+                qq|window.open('$url','_self')\n|,
+                qq|//-->\n</script>\n|,
+		qq|</center></body></html>\n|;
       exit(0);
 
    } else { # Password is INCORRECT
@@ -548,13 +408,13 @@ sub login {
 
       if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
          my $mailgid=getgrnam('mail');
-         set_euid_egid_umask($uuid, $mailgid, 0077);
+         set_euid_egid_umask($uuid, $mailgid, 0077);	
       }
 
       if ( $config{'use_homedirfolders'} ) {
          $folderdir = "$homedir/$config{'homedirfolderdirname'}";
       } else {
-         $folderdir = "$config{'ow_usersdir'}/$user";
+         $folderdir = "$config{'ow_etcdir'}/users/$user";
          $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
       }
 
@@ -569,13 +429,20 @@ sub login {
 
       sleep $config{'loginerrordelay'};	# delayed response
 
-      my $html=readtemplate("loginfailed.template");
-      $html = applystyle($html);
+      my $html = '';
+      printheader();
+      open (LOGINFAILED, "$config{'ow_etcdir'}/templates/$prefs{'language'}/loginfailed.template") or
+         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/loginfailed.template!");
+      while (<LOGINFAILED>) {
+         $html .= $_;
+      }
+      close (LOGINFAILED);
       $html =~ s/\@\@\@ERRORMSG\@\@\@/$errormsg/;
 
-      printheader();
+      $html = applystyle($html);
+      
       print $html;
-      printfooter(1);
+      printfooter();
       exit 0;
    }
 }
@@ -594,69 +461,23 @@ sub ip2hostname {
 }
 #################### END LOGIN #####################
 
-############### IS_SERVERDOMAIN_ALLOWED ###################
-sub is_serverdomain_allowed {
-   my $domain=$_[0];
-   if ($#{$config{'allowed_serverdomain'}}>=0) {
-      foreach my $token (@{$config{'allowed_serverdomain'}}) {
-         if (lc($token) eq 'all' || lc($domain) eq lc($token)) {
-            return 1;
-         } elsif (lc($token) eq 'none') {
-            return 0;
-         }
-      }
-      return 0;
-   } else {
-      return 1;
-   }
-}
-############### END IS_SERVERDOMAIN_ALLOWED ###################
-
-################ SEARCH_AND_CLEANOLDSESSIONS ##################
-# delete expired session files and
-# try to find old session that is still valid for the same user cookie
-sub search_and_cleanoldsessions {
-   my $oldcookie=$_[0];
-   my $oldsessionid="";
+################ CLEANUPOLDSESSIONS ##################
+sub cleanupoldsessions {
    my $sessionid;
-   my @delfiles;
-
-   opendir (SESSIONSDIR, "$config{'ow_sessionsdir'}") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}!");
+   opendir (SESSIONSDIR, "$config{'ow_etcdir'}/sessions") or
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions!");
    while (defined($sessionid = readdir(SESSIONSDIR))) {
       if ($sessionid =~ /^(.+\-session\-0.*)$/) {
          $sessionid = $1;
-         if ($sessionid =~ /^$loginname\-session\-0./) { # remove user old session if timeout
-            if ( -M "$config{'ow_sessionsdir'}/$sessionid" > $prefs{'sessiontimeout'}/60/24 ) {
-               writelog("session cleanup - $sessionid");
-               push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
-            } else {	# remove user old session from same client
-               open (SESSION, "$config{'ow_sessionsdir'}/$sessionid");
-               my $cookie = <SESSION>; chomp $cookie;
-               my $ip = <SESSION>; chomp $ip;
-               close (SESSION);
-               if ($ip eq get_clientip()) {
-                  if ($cookie eq $oldcookie && $oldcookie ne "") {
-                     $oldsessionid=$sessionid;
-                  } else {
-                     writelog("session cleanup - $sessionid");
-                     push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
-                  }
-               }
-            }
-         } else {	# remove others old session if more than 1 day
-            if ( -M "$config{'ow_sessionsdir'}/$sessionid" > 1 ) {
-               writelog("session cleanup - $sessionid");
-               push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
-            }
+         if ( -M "$config{'ow_etcdir'}/sessions/$sessionid" > $config{'sessiontimeout'}/60/24 ) {
+            writelog("session cleanup - $sessionid");
+            unlink "$config{'ow_etcdir'}/sessions/$sessionid";
          }
       }
    }
    closedir (SESSIONSDIR);
-   unlink(@delfiles) if ($#delfiles>=0);
-   return($oldsessionid);
 }
-############## END SEARCH_AND_CLEANOLDSESSIONS ################
+############## END CLEANUPOLDSESSIONS ################
 
 #################### RELEASEUPGRADE ####################
 # convert file format from old release for backward compatibility
@@ -664,8 +485,9 @@ sub releaseupgrade {
    my ($folderdir, $user_releasedate)=@_;
    my $content;
    my $rc_upgrade=0;
-   my ($_OFFSET, $_FROM, $_TO, $_DATE, $_SUBJECT, $_CONTENT_TYPE, $_STATUS, $_SIZE, $_REFERENCES, $_CHARSET)
-       =(0,1,2,3,4,5,6,7,8,9);
+   local ($_OFFSET, $_FROM, $_TO, $_DATE, $_SUBJECT, $_CONTENT_TYPE, $_STATUS, $_SIZE, $_REFERENCES)
+       =(0,1,2,3,4,5,6,7,8);
+
 
    if ( $user_releasedate lt "20011101" ) {
       if ( -f "$folderdir/.filter.book" ) {
@@ -755,8 +577,9 @@ sub releaseupgrade {
       opendir (FOLDERDIR, "$folderdir") or
          openwebmailerror("$lang_err{'couldnt_open'} $folderdir");
       while (defined($file = readdir(FOLDERDIR))) {
-         if ($file=~/^(\..+\.cache)$/) {
-            $file="$folderdir/$1";
+         if ($file=~/^\..*.cache$/) {
+            $file="$folderdir/$file";
+            ($file=~/^(.*)$/) && ($file=$1);
             push(@cachefiles, $file);
          }
       }
@@ -765,7 +588,7 @@ sub releaseupgrade {
          writehistory("release upgrade - $folderdir/*.cache by 20011216");
          writelog("release upgrade - $folderdir/*.cache by 20011216");
          # remove old .cache since its format is not compatible with new one
-         unlink(@cachefiles);
+         unlink(@cachefiles); 
       }
    }
 
@@ -776,27 +599,28 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
-         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
+
+         next if ( ! -f "$headerdb$config{'dbm_ext'}");
 
          filelock($folderfile, LOCK_SH);
          open (FOLDER, $folderfile);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
+         dbmopen (%HDB, $headerdb, undef);
 
          if ( $HDB{'METAINFO'} eq metainfo($folderfile) ) { # upgrade only if hdb is uptodate
             @messageids=keys %HDB;
             foreach my $id (@messageids) {
-               next if ( $id eq 'METAINFO'
-                || $id eq 'NEWMESSAGES'
-                || $id eq 'INTERNALMESSAGES'
-                || $id eq 'ALLMESSAGES'
+               next if ( $id eq 'METAINFO' 
+                || $id eq 'NEWMESSAGES' 
+                || $id eq 'INTERNALMESSAGES' 
+                || $id eq 'ALLMESSAGES' 
                 || $id eq "" );
 
                @attr=split( /@@@/, $HDB{$id} );
 
                next if ( ($attr[$_CONTENT_TYPE] eq '' ||
                           $attr[$_CONTENT_TYPE] eq 'N/A' ||
-                          $attr[$_CONTENT_TYPE] =~ /^text/i)
+                          $attr[$_CONTENT_TYPE] =~ /^text/i) 
                        && $attr[$_SIZE]<4096 );
 
                next if ($attr[$_STATUS] =~ /T/i);
@@ -826,7 +650,7 @@ sub releaseupgrade {
             }
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
          filelock($folderfile, LOCK_UN);
       }
       writehistory("release upgrade - $folderdir/* by 20020108.02");
@@ -840,17 +664,18 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
-         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
+         next if ( ! -f "$headerdb$config{'dbm_ext'}");
+
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
+         dbmopen (%HDB, $headerdb, undef);
 
          @messageids=keys %HDB;
          foreach my $id (@messageids) {
-            next if ( $id eq 'METAINFO'
-             || $id eq 'NEWMESSAGES'
-             || $id eq 'INTERNALMESSAGES'
-             || $id eq 'ALLMESSAGES'
+            next if ( $id eq 'METAINFO' 
+             || $id eq 'NEWMESSAGES' 
+             || $id eq 'INTERNALMESSAGES' 
+             || $id eq 'ALLMESSAGES' 
              || $id eq "" );
 
             @attr=split( /@@@/, $HDB{$id} );
@@ -861,12 +686,12 @@ sub releaseupgrade {
             } elsif ($d[2]<=1900) {
                $d[2]+=1900;
             }
-            $attr[$_DATE]=sprintf("%4d%02d%02d%02d%02d%02d",
+            $attr[$_DATE]=sprintf("%4d%02d%02d%02d%02d%02d", 
 					$d[2],$d[0],$d[1], $d[3],$d[4],$d[5]);
             $HDB{$id}=join('@@@', @attr);
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
 
          my $cachefile="$headerdb.cache";
          ($cachefile =~ /^(.+)$/) && ($cachefile = $1);  # untaint ...
@@ -876,97 +701,7 @@ sub releaseupgrade {
       writelog("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20020108.02");
    }
 
-   if ( $user_releasedate lt "20020601" ) {
-      my $timeoffset=gettimeoffset();
-      $timeoffset=~s/\+/-/ || $timeoffset=~s/\-/+/;	# switch +/-
-      my (@validfolders, $folderusage);
-      getfolders(\@validfolders, \$folderusage);
-
-      foreach my $foldername (@validfolders) {
-         my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
-         my (%HDB, @messageids, @attr);
-         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
-
-         filelock($folderfile, LOCK_SH);
-         open (FOLDER, $folderfile);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
-
-         @messageids=keys %HDB;
-         foreach my $id (@messageids) {
-            next if ( $id eq 'METAINFO'
-                   || $id eq 'NEWMESSAGES'
-                   || $id eq 'INTERNALMESSAGES'
-                   || $id eq 'ALLMESSAGES'
-                   || $id eq "" );
-            my ($buff, $delimiter, $datefield, $dateserial);
-            @attr=split( /@@@/, $HDB{$id} );
-            seek(FOLDER, $attr[$_OFFSET], 0);
-            if (length($attr[$_FROM].$attr[$_TO].$attr[$_SUBJECT].$attr[$_CONTENT_TYPE].$attr[$_REFERENCES])>384) {
-                read(FOLDER, $buff, 2048);
-            } else {
-                read(FOLDER, $buff, 1024);
-            }
-            if ( $buff =~ /^From (.+?)\n/ims) {
-               $delimiter=$1;
-               if ( $buff =~ /\nDate: (.+?)\n/ims ) {
-                  $datefield=$1;
-               }
-               my $dateserial=datefield2dateserial($datefield);
-               my $deliserial=delimiter2dateserial($delimiter, $config{'deliver_use_GMT'});
-               if ($dateserial eq "" ||
-                   ($deliserial ne "" && dateserial2daydiff($dateserial)-dateserial2daydiff($deliserial)>1) ) {
-                  $dateserial=$deliserial; # use receiving time if sending time is newer than receiving time
-               }
-               $dateserial=gmtime2dateserial() if ($dateserial eq "");
-               $attr[$_DATE]=$dateserial;
-            } else {
-               $attr[$_DATE]=add_dateserial_timeoffset($attr[$_DATE], $timeoffset);
-            }
-            $HDB{$id}=join('@@@', @attr);
-         }
-         dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-         filelock($folderfile, LOCK_UN);
-      }
-      writehistory("release upgrade - $folderdir/* by 20020601");
-      writelog("release upgrade - $folderdir/* by 20020601");
-   }
-
-   if ( $user_releasedate lt "20021111" ) {
-      my (@validfolders, $folderusage);
-      getfolders(\@validfolders, \$folderusage);
-
-      foreach my $foldername (@validfolders) {
-         my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
-         my (%HDB, @messageids, @attr);
-         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
-
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
-
-         @messageids=keys %HDB;
-         foreach my $id (@messageids) {
-            next if ( $id eq 'METAINFO'
-             || $id eq 'NEWMESSAGES'
-             || $id eq 'INTERNALMESSAGES'
-             || $id eq 'ALLMESSAGES'
-             || $id eq "" );
-            @attr=split( /@@@/, $HDB{$id} );
-            if ( $attr[$_CHARSET] eq "" &&
-                 $attr[$_CONTENT_TYPE]=~/charset="?([^\s"';]*)"?\s?/i) {
-               $attr[$_CHARSET]=$1;
-               $HDB{$id}=join('@@@', @attr);
-            }
-         }
-         dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
-      }
-      writehistory("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20021111.02");
-      writelog("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20021111.02");
-   }
-
-   if ( $user_releasedate lt "20021116" ) {
+   if ( $user_releasedate lt "20020220" ) {
       $rc_upgrade=1;	# .openwebmailrc upgrade will be requested
    }
 
